struct VtxInput
{
	float4 	Position	:	POSITION0;
	float3 	Normal		:	NORMAL0;
	float2 	TextureUV	:	TEXCOORD0;
	float3	Tangent		:	TANGENT0;
};

struct VtxOutput
{
	float4 	Position	:	POSITION0;
	float3 	Vertex		:	POSITION1;
	float3 	Normal		:	NORMAL0;
	float2 	TextureUV	:	TEXCOORD0;
	float3	Tangent		:	TANGENT0;
	float3	Binormal	:	TANGENT1;
	float3	Eye			:	NORMAL1;
};

uniform 	float4x4 	c3d_mtxProjection;
uniform 	float4x4 	c3d_mtxProjectionModelView;
uniform 	float4x4 	c3d_mtxModel;
uniform 	float4x4 	c3d_mtxView;
uniform 	float4x4 	c3d_mtxNormal;
uniform 	float4x4 	c3d_mtxModelView;
uniform 	float4 		c3d_v4AmbientLight;
uniform 	float4 		c3d_v4MatAmbient;
uniform 	float4 		c3d_v4MatEmissive;
uniform 	float4 		c3d_v4MatDiffuse;
uniform 	float4 		c3d_v4MatSpecular;
uniform 	float 		c3d_fMatShininess;

uniform 	int 		c3d_iLightsCount;

texture2D 				lightsTexture;
texture2D				diffuseTexture;
texture2D				normalTexture;
texture2D				specularTexture;
texture2D				opacityTexture;

sampler2D LightsSampler : register(s0)
{
    Texture = <lightsTexture>;
    MagFilter = NEAREST;
    MinFilter = NEAREST;
    MipFilter = NEAREST;
};

sampler DiffuseTextureSampler : register(s1)
{
	Texture = <diffuseTexture>;
    AddressU  = WRAP;
	AddressV  = WRAP;
    AddressW  = WRAP;
    MipFilter = NONE;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler NormalTextureSampler : register(s2)
{
	Texture = <normalTexture>;
    AddressU  = WRAP;
	AddressV  = WRAP;
    AddressW  = WRAP;
    MipFilter = NONE;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler SpecularTextureSampler : register(s3)
{
	Texture = <specularTexture>;
    AddressU  = WRAP;
	AddressV  = WRAP;
    AddressW  = WRAP;
    MipFilter = NONE;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler OpacityTextureSampler : register(s4)
{
	Texture = <opacityTexture>;
    AddressU  = WRAP;
	AddressV  = WRAP;
    AddressW  = WRAP;
    MipFilter = NONE;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

struct Light
{
	float4 		m_v4Ambient;
	float4 		m_v4Diffuse;
	float4 		m_v4Specular;
	float4		m_v4Position;
	int 		m_iType;
	float3		m_v3Attenuation;
	float4x4	m_mtx4Orientation;
	float		m_fExponent;
	float		m_fCutOff;
};

Light GetLight( int i )
{
	Light l_lightReturn;
	l_lightReturn.m_v4Ambient		= tex2D( LightsSampler, float2( (i * 0.1) + (0 * 0.01) + 0.005, 0.0 ) );
	l_lightReturn.m_v4Diffuse		= tex2D( LightsSampler, float2( (i * 0.1) + (1 * 0.01) + 0.005, 0.0 ) );
	l_lightReturn.m_v4Specular		= tex2D( LightsSampler, float2( (i * 0.1) + (2 * 0.01) + 0.005, 0.0 ) );
	float4 	l_v4Position			= tex2D( LightsSampler, float2( (i * 0.1) + (3 * 0.01) + 0.005, 0.0 ) );
	l_lightReturn.m_v3Attenuation	= tex2D( LightsSampler, float2( (i * 0.1) + (4 * 0.01) + 0.005, 0.0 ) ).xyz;
	float4 	l_v4A					= tex2D( LightsSampler, float2( (i * 0.1) + (5 * 0.01) + 0.005, 0.0 ) );
	float4 	l_v4B					= tex2D( LightsSampler, float2( (i * 0.1) + (6 * 0.01) + 0.005, 0.0 ) );
	float4 	l_v4C					= tex2D( LightsSampler, float2( (i * 0.1) + (7 * 0.01) + 0.005, 0.0 ) );
	float4 	l_v4D					= tex2D( LightsSampler, float2( (i * 0.1) + (8 * 0.01) + 0.005, 0.0 ) );
	float2	l_v2Spot				= tex2D( LightsSampler, float2( (i * 0.1) + (9 * 0.01) + 0.005, 0.0 ) ).xy;
	l_lightReturn.m_v4Position		= float4( l_v4Position.x, l_v4Position.y, l_v4Position.z, 0.0 );
	l_lightReturn.m_iType			= int( l_v4Position.w );
	l_lightReturn.m_mtx4Orientation	= float4x4( l_v4A, l_v4B, l_v4C, l_v4D );
	l_lightReturn.m_fExponent		= l_v2Spot.x;
	l_lightReturn.m_fCutOff			= l_v2Spot.x;
	return l_lightReturn;
}

VtxOutput mainVx( in VtxInput p_input )
{
	float3x3 l_mtxNormal = (float3x3)c3d_mtxNormal;
	VtxOutput l_output;
	l_output.Position	= mul( p_input.Position, c3d_mtxProjectionModelView );
	l_output.Normal 	= normalize( mul( p_input.Normal, l_mtxNormal ) );
	l_output.TextureUV	= p_input.TextureUV;
	l_output.Tangent 	= normalize( mul( p_input.Tangent, l_mtxNormal ) );
	l_output.Vertex		= mul( p_input.Position, c3d_mtxModelView ).xyz;
	l_output.Binormal	= cross( l_output.Tangent, l_output.Normal );

	float3 tmpVec = -l_output.Vertex;
	l_output.Eye.x = dot( tmpVec, l_output.Tangent	);
	l_output.Eye.y = dot( tmpVec, l_output.Binormal	);
	l_output.Eye.z = dot( tmpVec, l_output.Normal	);

	return l_output;
}

float4 mainPx( in VtxOutput p_input )	:	COLOR0
{
	Light	l_light;
	float	l_fInvRadius;
	float3	l_v3Tmp;
	float3 	l_v3Light;
	float 	l_fSqrLength;
	float 	l_fAttenuation;
	float3 	l_v3EyeVec;
	float4	l_v4Texture;
	float4	l_v4Normal;
	float3	l_v3Normal;
	float4	l_v4Ambient		= float4( 0, 0, 0, 0 );
	float4	l_v4Diffuse		= float4( 0, 0, 0, 0 );
	float4	l_v4Specular	= float4( 0, 0, 0, 0 );
	float4	l_v4MapSpec;
	float4	l_v4Opacity;
	
	l_fInvRadius	= 0.02;
	l_v3EyeVec		= normalize( p_input.Eye );
	l_v4Texture		= tex2D( DiffuseTextureSampler, p_input.TextureUV );
	l_v4Normal		= tex2D( NormalTextureSampler, p_input.TextureUV );
	l_v4MapSpec		= tex2D( SpecularTextureSampler, p_input.TextureUV );
	l_v4Opacity		= tex2D( OpacityTextureSampler, p_input.TextureUV );
	l_v3Normal		= normalize( l_v4Normal.xyz * 2.0 - 1.0 );
	
	for( int i = 0 ; i < 3 ; i++ )
	{
		l_light			= GetLight( i );
		l_v3Tmp			= normalize( mul( l_light.m_v4Position, c3d_mtxModelView ).xyz - p_input.Vertex );
		l_v3Light.x 	= dot( l_v3Tmp, p_input.Tangent		);
		l_v3Light.y 	= dot( l_v3Tmp, p_input.Binormal	);
		l_v3Light.z 	= dot( l_v3Tmp, p_input.Normal		);
		l_fSqrLength	= dot( l_v3Light, l_v3Light );
		l_v3Light		= l_v3Light * rsqrt( l_fSqrLength );
		l_fAttenuation	= clamp( 1.0 - l_fInvRadius * sqrt( l_fSqrLength ), 0.0, 1.0 );
		l_v4Ambient		+= l_light.m_v4Ambient	* c3d_v4MatAmbient	* l_fAttenuation;
		l_v4Diffuse		+= l_light.m_v4Diffuse	* c3d_v4MatDiffuse	* l_fAttenuation	* max( dot( l_v3Light, l_v3Normal ), 0.0 );
		l_v4Specular	+= l_light.m_v4Specular * c3d_v4MatSpecular	* l_fAttenuation	* l_v4MapSpec * pow( clamp( dot( reflect( -l_v3Light, l_v3Normal ), l_v3EyeVec ), 0.0, 1.0 ), c3d_fMatShininess );
	}

	return float4( (l_v4Ambient * l_v4Texture + l_v4Diffuse * l_v4Texture + l_v4Specular).xyz, l_v4Opacity.r );
}
